Attribute VB_Name = "CircleStuff"
Option Explicit

Public Const NAN As Single = 1E+20

' Find the points where the two circles intersect.
Public Function FindCircleCircleIntersections( _
    ByVal center0 As PointF, ByVal radius0 As Single, _
    ByVal center1 As PointF, ByVal radius1 As Single, _
    ByRef intersection1 As PointF, _
    ByRef intersection2 As PointF) As Integer
Dim dx As Single
Dim dy As Single
Dim dist As Double
Dim a As Double
Dim h As Double
Dim cx2 As Double
Dim cy2 As Double

    ' Find the distance between the centers.
    dx = center0.X - center1.X
    dy = center0.Y - center1.Y
    dist = Sqr(dx * dx + dy * dy)

    ' See how many solutions there are.
    If (dist > radius0 + radius1) Then
        ' No solutions, the circles are too far apart.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
        Exit Function
    ElseIf (dist < Math.Abs(radius0 - radius1)) Then
        ' No solutions, one circle contains the other.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
    ElseIf ((dist = 0) And (radius0 = radius1)) Then
        ' No solutions, the circles coincide.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
    Else
        ' Find a and h.
        a = (radius0 * radius0 - _
            radius1 * radius1 + dist * dist) / (2 * dist)
        h = Sqr(radius0 * radius0 - a * a)

        ' Find P2.
        cx2 = center0.X + a * (center1.X - center0.X) / dist
        cy2 = center0.Y + a * (center1.Y - center0.Y) / dist

        ' Get the points P3.
        Set intersection1 = New PointF
        Set intersection2 = New PointF
        intersection1.X = CSng(cx2 + h * (center1.Y - center0.Y) / dist)
        intersection1.Y = CSng(cy2 - h * (center1.X - center0.X) / dist)
        intersection2.X = CSng(cx2 - h * (center1.Y - center0.Y) / dist)
        intersection2.Y = CSng(cy2 + h * (center1.X - center0.X) / dist)

        ' See if we have 1 or 2 solutions.
        If (dist = radius0 + radius1) Then
            FindCircleCircleIntersections = 1
        Else
            FindCircleCircleIntersections = 2
        End If
        Exit Function
    End If
End Function

' Find the tangent points for this circle and external point.
' Return true if we find the tangents, false if the point is
' inside the circle.
Public Function FindTangents( _
    ByVal center As PointF, ByVal radius As Single, _
    ByVal pt As PointF, _
    ByRef pt1 As PointF, ByRef pt2 As PointF) As Boolean
Dim dx As Double
Dim dy As Double
Dim D_squared As Double
Dim L As Double

    ' Find the distance squared from the
    ' external point to the circle's center.
    dx = center.X - pt.X
    dy = center.Y - pt.Y
    D_squared = dx * dx + dy * dy
    If (D_squared < radius * radius) Then
        pt1.X = -1
        FindTangents = False
        Exit Function
    End If

    ' Find the distance from the external point
    ' to the tangent points.
    L = Sqr(D_squared - radius * radius)

    ' Find the points of intersection between
    ' the original circle and the circle with
    ' center external_point and radius dist.
    FindCircleCircleIntersections _
        center, radius, pt, CSng(L), pt1, pt2

    FindTangents = True
End Function

' Find the tangent points for these two circles.
' Return the number of tangents: 4, 2, or 0.
Public Function FindCircleCircleTangents( _
    ByVal c1 As PointF, ByVal radius1 As Single, ByVal c2 As PointF, ByVal radius2 As Single, _
    ByRef outer1_p1 As PointF, ByRef outer1_p2 As PointF, _
    ByRef outer2_p1 As PointF, ByRef outer2_p2 As PointF, _
    ByRef inner1_p1 As PointF, ByRef inner1_p2 As PointF, _
    ByRef inner2_p1 As PointF, ByRef inner2_p2 As PointF) _
As Integer

Dim radius2a As Single
Dim v1x As Single
Dim v1y As Single
Dim v1_length As Single
Dim v2x As Single
Dim v2y As Single
Dim v2_length As Single
Dim dx As Single
Dim dy As Single
Dim dist As Double
Dim radius1a As Single
    
    ' Make sure radius1 <= radius2.
    If (radius1 > radius2) Then
        ' Call this method switching the circles.
        FindCircleCircleTangents = _
            FindCircleCircleTangents( _
                c2, radius2, c1, radius1, _
                outer1_p2, outer1_p1, _
                outer2_p2, outer2_p1, _
                inner1_p2, inner1_p1, _
                inner2_p2, inner2_p1)
        Exit Function
    End If

    ' Initialize the return values in case
    ' some tangents are missing.
    Set outer1_p1 = New PointF
    Set outer1_p2 = New PointF
    Set outer2_p1 = New PointF
    Set outer2_p2 = New PointF
    Set inner1_p1 = New PointF
    Set inner1_p2 = New PointF
    Set inner2_p1 = New PointF
    Set inner2_p2 = New PointF

    ' ***************************
    ' * Find the outer tangents *
    ' ***************************
    radius2a = radius2 - radius1
    If (Not FindTangents(c2, radius2a, c1, outer1_p2, outer2_p2)) Then
        ' There are no tangents.
        FindCircleCircleTangents = 0
        Exit Function
    End If

    ' Get the vector perpendicular to the
    ' first tangent with length radius1.
    v1x = -(outer1_p2.Y - c1.Y)
    v1y = outer1_p2.X - c1.X
    v1_length = CSng(Sqr(v1x * v1x + v1y * v1y))
    v1x = v1x * radius1 / v1_length
    v1y = v1y * radius1 / v1_length
    ' Offset the tangent vector's points.
    outer1_p1.X = c1.X + v1x
    outer1_p1.Y = c1.Y + v1y
    outer1_p2.X = outer1_p2.X + v1x
    outer1_p2.Y = outer1_p2.Y + v1y

    ' Get the vector perpendicular to the
    ' second tangent with length radius1.
    v2x = outer2_p2.Y - c1.Y
    v2y = -(outer2_p2.X - c1.X)
    v2_length = CSng(Sqr(v2x * v2x + v2y * v2y))
    v2x = v2x * radius1 / v2_length
    v2y = v2y * radius1 / v2_length
    ' Offset the tangent vector's points.
    outer2_p1.X = c1.X + v2x
    outer2_p1.Y = c1.Y + v2y
    outer2_p2.X = outer2_p2.X + v2x
    outer2_p2.Y = outer2_p2.Y + v2y

    ' If the circles intersect, then there are no inner tangents.
    dx = c2.X - c1.X
    dy = c2.Y - c1.Y
    dist = Sqr(dx * dx + dy * dy)
    If (dist <= radius1 + radius2) Then
        FindCircleCircleTangents = 2
        Exit Function
    End If

    ' ***************************
    ' * Find the inner tangents *
    ' ***************************
    radius1a = radius1 + radius2
    FindTangents c1, radius1a, c2, inner1_p2, inner2_p2

    ' Get the vector perpendicular to the
    ' first tangent with length radius2.
    v1x = inner1_p2.Y - c2.Y
    v1y = -(inner1_p2.X - c2.X)
    v1_length = CSng(Sqr(v1x * v1x + v1y * v1y))
    v1x = v1x * radius2 / v1_length
    v1y = v1y * radius2 / v1_length
    ' Offset the tangent vector's points.
    inner1_p1.X = c2.X + v1x
    inner1_p1.Y = c2.Y + v1y
    inner1_p2.X = inner1_p2.X + v1x
    inner1_p2.Y = inner1_p2.Y + v1y

    ' Get the vector perpendicular to the
    ' second tangent with length radius2.
    v2x = -(inner2_p2.Y - c2.Y)
    v2y = inner2_p2.X - c2.X
    v2_length = CSng(Sqr(v2x * v2x + v2y * v2y))
    v2x = v2x * radius2 / v2_length
    v2y = v2y * radius2 / v2_length
    ' Offset the tangent vector's points.
    inner2_p1.X = c2.X + v2x
    inner2_p1.Y = c2.Y + v2y
    inner2_p2.X = inner2_p2.X + v2x
    inner2_p2.Y = inner2_p2.Y + v2y

    FindCircleCircleTangents = 4
End Function


